
/* PAPER: DSV (2018) - REPLICATION OF AMIHUD (2002) (A02) FOR Critical Finance Review */

* Libary reference name: "input" is where the initial input datasets are located;
libname input 'C:\Users\Anna\Dropbox\Code and Data for CFR - DSV 2018\Input'; run;

/*==================================================================================*/

								/*PART 3: ROBUSTNESS*/

					   	 /*Performed on full sample period*/

/*==================================================================================*/

/* ======================= Daily data: ILLIQ calculation =========================== */

/* input.CRSP_daily_stock_NYSE is the CRSP Daily Stock File 
(https://wrds-web.wharton.upenn.edu/wrds/ds/crsp/stock_a/dsf.cfm?navId=128) 
(note: this file already only contains NYSE stocks due to its size) */

data work.CRSP_daily_stock_NYSE;
	set input.CRSP_daily_stock_NYSE;
	where Year >= 1962 and year <= 2015;
run;

data work.CRSP_daily_stock_NYSE;
	set work.CRSP_daily_stock_NYSE;
	where (EXCHCD = 1 or PRIMEXCH = 'N'); /*no change as already filtered to NYSE*/
run;

/*Delete variables with missing observations for RET, PRC (including 0 values) and VOL 
  as all used in ILLIQ calculation. */

proc sort data=work.CRSP_daily_stock_NYSE; by permno date; run;

data work.CRSP_daily_stock_NYSE;
	set work.CRSP_daily_stock_NYSE;
	by permno date;
	lag_ret = LAG(ret); 
	if first.permno then do;
		lag_ret = .; 
		end;
run;

data work.CRSP_daily_stock_NYSE;
	set work.CRSP_daily_stock_NYSE;
	if lag_ret = . then delete; /************** ROBUSTNESS CHECK #2 *************/
run;

data work.CRSP_daily_stock_NYSE;
	set work.CRSP_daily_stock_NYSE;
	where RET is not missing and abs(PRC) > 0 and VOL > 0;
run;

/* ========================== CPI Adjustment Data: ROBUSTNESS CHECK #7 ========================== */

/* match by year month to CPI data */

data work.CRSP_daily_stock_NYSE;
	set work.CRSP_daily_stock_NYSE;
	month = month(date);
		Month_num=put(Month,z2.);
		YearMonth=cat(year,Month_num);
		YearMonth_num=input(YearMonth, 6.);
run;

data work.CRSP_daily_stock_NYSE;
	set work.CRSP_daily_stock_NYSE;
	drop month Month_num YearMonth;
run;

proc sort data=work.CRSP_daily_stock_NYSE; by yearmonth_num; run;

data work.CRSP_daily_stock_NYSE;
	merge 	work.CRSP_daily_stock_NYSE (in=a)
			input.CPI_DATA (in=b);
	by YearMonth_num;
	if a;
run;

proc sort data=work.CRSP_daily_stock_NYSE; by permno date; run;

/* create dollar volume (DOLLAR_VOL) variable and absolute return/dollar volume variable 
for ILLIQ calculation; also MKT_CAP for data sample requirement *5* below */

data work.CRSP_daily_stock_NYSE;
	set work.CRSP_daily_stock_NYSE;
	DOLLAR_VOL=abs(PRC)*VOL;
	Abs_ret_per_dollar_vol=abs(ret)/DOLLAR_VOL;
	DOLLAR_VOL_INF=abs(PRC)*VOL*CPI_END_2015_DEC; /*CPI_END_2015_DEC is DSV variable INFyd */
	Abs_ret_per_dol_vol_INF=abs(ret)/DOLLAR_VOL_INF;
	MKT_CAP=abs(PRC)*SHROUT;
run;

/* Calculate Inflation adjusted ILLIQ for each stock i over each year y */

proc sql;
create table work.ILLIQ as
select distinct permno, year, 
	count(/*Abs_ret_per_dollar_vol*/Abs_ret_per_dol_vol_INF) as num_D, 					
	sum(/*Abs_ret_per_dollar_vol*/Abs_ret_per_dol_vol_INF) as Sum_Abs_ret_p_dollar_vol  
from work.CRSP_daily_stock_NYSE 
group by permno, year
;

data work.ILLIQ;
	set work.ILLIQ;
	ILLIQ=Sum_Abs_ret_p_dollar_vol/num_D; /************** ROBUSTNESS CHECK #7 *************/
run;

/* ------------Additional data requirements------------ */

/* Additional data requirements:
*2* The stock has return data for more than 200 days during year y-1 (requirement i p.36)
*3* The stock must be listed at the end of year y-1 (requirement i p.36)
*/

/* merge in input.delist, which is the CRSP stock/events delist file 
(https://wrds-web.wharton.upenn.edu/wrds/ds/crsp/stock_a/delist.cfm?navId=129)*/

data delist_date; set input.delist; keep permno DLSTDT; run;

data delist_date; 	
	set delist_date; 
	IF DLSTDT='31Dec2015'd THEN Year_Delist=.; 
	else Year_Delist = year(DLSTDT);
run;

Data ILLIQ; merge ILLIQ (in=a) delist_date (in=b);	by permno; if a; run;

*Impose number of days and listing restriction;

data work.ILLIQ_v2;
	set work.ILLIQ;
	if Year_Delist = . then Delist_status = "N";
	else if Year_Delist > year then Delist_status = "N";
	else Delist_status = "Y";
run;

data work.ILLIQ_v2;
	set work.ILLIQ_v2;
	where num_D > 200 
		  and Delist_status = "N";
run;

/* Additional Data requirements:
*4* The stock price > $5 at end of year y-1 (requirement ii p.36) 
*5* The stock has data on market capitalisation at the end of year y-1 (requirement iii p.37)
*/

/* proc sort daily data to keep the last variables at year end for requirement *4*, *5* */

data work.ILLIQ_requirements;
	set input.CRSP_daily_stock_NYSE;
	keep permno year date PRC SHROUT SHRCD EXCHCD PRIMEXCH year MKT_CAP;
	Year=year(DATE);
	MKT_CAP=abs(prc)*SHROUT;
run;

proc sort data=ILLIQ_requirements;
	by permno year descending date;
run;

proc sort data=ILLIQ_requirements nodupkey;
	by permno year;
run;

data work.ILLIQ_v2;
	merge 	work.ILLIQ_v2 (in=a)
			work.ILLIQ_requirements (in=b);
	by permno year;
	if a;
run;

data work.ILLIQ_v2;
	set work.ILLIQ_v2;
	where EXCHCD = 1 
		  and abs(PRC) > 0 /************** ROBUSTNESS CHECK #6: keep penny stocks *************/ 
		  and MKT_CAP > 0 
		  and SHRCD < 20; /************** ROBUSTNESS CHECK #1: restrict share code *************/
run;

proc freq data=work.ILLIQ_v2 nlevels;
	tables SHRCD;
run;



/* Additional data requirements:
*6* ROBUSTNESS CHECK #5: WINDSORIZE (not trim) stocks with ILLIQ in year y-1 at the highest and lowest 1%*/
  
%macro Winsorize_Truncate(dsetin = work.ILLIQ_v2, 
                          dsetout = work.ILLIQ_trimmed, 
                          byvar = year, 
                          vars = ILLIQ, 
                          type = w, /************** ROBUSTNESS CHECK #5 *************/
                          pctl = 1 99);


    %if &dsetout = %then %let dsetout = &dsetin;
    
    %let varL=;
    %let varH=;
    %let xn=1;

    %do %until (%scan(&vars,&xn)= );
        %let token = %scan(&vars,&xn);
        %let varL = &varL &token.L;
        %let varH = &varH &token.H;
        %let xn = %EVAL(&xn + 1);
    %end;

    %let xn = %eval(&xn-1);

    data xtemp;
        set &dsetin;

    %let dropvar = ;
    %if &byvar = none %then %do;
        data xtemp;
            set xtemp;
            xbyvar = 1;

        %let byvar = xbyvar;
        %let dropvar = xbyvar;
    %end;

    proc sort data = xtemp;
        by &byvar;

    /*compute percentage cutoff values*/
    proc univariate data = xtemp noprint;
        by &byvar;
        var &vars;
        output out = xtemp_pctl PCTLPTS = &pctl PCTLPRE = &vars PCTLNAME = L H;

    data &dsetout;
        merge xtemp xtemp_pctl; /*merge percentage cutoff values into main dataset*/
        by &byvar;
        array trimvars{&xn} &vars;
        array trimvarl{&xn} &varL;
        array trimvarh{&xn} &varH;

        do xi = 1 to dim(trimvars);
            /*winsorize variables*/
            %if &type = W %then %do;
                if trimvars{xi} ne . then do;
                    if (trimvars{xi} < trimvarl{xi}) then trimvars{xi} = trimvarl{xi};
                    if (trimvars{xi} > trimvarh{xi}) then trimvars{xi} = trimvarh{xi};
                end;
            %end;
            /*truncate variables*/
            %else %do;
                if trimvars{xi} ne . then do;
                    /*insert .T code if value is truncated*/
                    if (trimvars{xi} < trimvarl{xi}) then trimvars{xi} = .T;
                    if (trimvars{xi} > trimvarh{xi}) then trimvars{xi} = .T;
                end;
            %end;
        end;
        drop &varL &varH &dropvar xi;

    /*delete temporary datasets created during macro execution*/
    proc datasets library=work nolist;
        delete xtemp xtemp_pctl; quit; run;

%mend Winsorize_Truncate;

/*invoke macro to winsorize */

%Winsorize_Truncate(dsetin=work.ILLIQ_v2, dsetout=work.ILLIQ_trimmed, byvar=year, vars=ILLIQ, 
	type = W, pctl=1 99); /************** ROBUSTNESS CHECK #5: gain 2% obs p.a. *************/
run;

data work.ILLIQ_trimmed;
	set work.ILLIQ_trimmed;
	where ILLIQ is not missing; 
	ILLIQ_1000000=ILLIQ*1000000; 
run;

* Delete datasets no longer needed;
proc datasets library=work; delete ILLIQ ILLIQ_V2; run;


/*********************** other variables ***********************

		*** ILLIQMA = ILLIQ/AILLIQ, where AILLIQ = average ILLIQ across 
					  stocks in year y ***/

proc sort data=ILLIQ_trimmed; by year permno; run;

proc sql;
create table work.AILLIQ as
select distinct year, 
	sum(ILLIQ)/count(ILLIQ) as AILLIQ,
	count(ILLIQ) as num_stocks
from work.ILLIQ_trimmed 
group by year
;

data ILLIQ_trimmed; merge ILLIQ_trimmed (in=a) AILLIQ (in=b); by year; if a; run;
data ILLIQ_trimmed; set ILLIQ_trimmed; ILLIQMA=ILLIQ/AILLIQ; run;


		/*** R100: return of stock i during the last 100 days of year y
			R100YR: return of stock i over the rest of the period ***

The last 100 days of every year spans 23 September - 31 December inclusive.
- create variable r100_BE = "E" if the day is in the last 100 days of each year and "B" otherwise
- create variable ln_RET_plus_1 = ln(1+RET) for cumulative return calculation
- split the sample into two: r100_BE = "E" and r100_BE = "B"
- in each sub-sample, create the two annual variables R100 and R100YR by calculating a
  cumulative return for each permno by year
- merge the two files into main dataset*/

proc means data=work.CRSP_daily_stock_NYSE n nmiss min max mean median; var ret; run;

data work.r100_prep;
	set work.CRSP_daily_stock_NYSE;
	keep permno date year RET r100_BE ln_RET_plus_1;
	ln_RET_plus_1 = log(RET+1);
	if month(date) > 9 then r100_BE = 'E';
	else if month(date)=9 and day (date) >= 23 then r100_BE = 'E';
	else r100_BE = 'B';
run;

data work.r100_bgn; set work.r100_prep; where r100_BE = 'B'; run;
data work.r100_end; set work.r100_prep; where r100_BE = 'E'; run;

proc sort data=r100_end; by permno date; run;
proc sort data=r100_bgn; by permno date; run;

proc sql;
create table work.r100 as
select distinct permno, year, 
	count(permno) as num_days_E,
	(exp(sum(ln_RET_plus_1))-1) as R100
from work.r100_end 
group by permno, year
;

proc sql;
create table work.r100yr as
select distinct permno, year, 
	count(permno) as num_days_B,
	(exp(sum(ln_RET_plus_1))-1) as R100YR
from work.r100_bgn 
group by permno, year
;

proc sort data=ILLIQ_trimmed; by permno year; run;
Data ILLIQ_trimmed; merge ILLIQ_trimmed (in=a) r100 (in=b);	by permno year; if a; run;
Data ILLIQ_trimmed; merge ILLIQ_trimmed (in=a) r100yr (in=b);	by permno year; if a; run;
data ILLIQ_trimmed; set ILLIQ_trimmed; drop num_days_B num_days_E; run;


		/*** ln_size_mill: the natural log of market capitalisation, in millions ***/

data work.ILLIQ_trimmed;
	set work.ILLIQ_trimmed;
	MKT_CAP_MILLIONS=(MKT_CAP*1000)/1000000; /* CRSP shares outstanding in '000. Amihud Table 1 in mill*/
	ln_size_mill=log(MKT_CAP_MILLIONS);
run;


		/*** stdev_ret: Standard deviation of daily return on stock i in year y, 
			 stdev_ret_100 = stdev_ret multiplied by 10^2 as per Amihud p. 38 ***/

proc sql;
create table work.std_dev as
select distinct permno, year, 
	std(ret) as stdev_ret 
from work.CRSP_daily_stock_NYSE 
group by permno, year
;
Data ILLIQ_trimmed; merge ILLIQ_trimmed (in=a) std_dev (in=b);	by permno year; if a; run;
data work.ILLIQ_trimmed; set work.ILLIQ_trimmed; STDEV_RET_100=stdev_ret*100; run;


		/*** DIVYLD: the dividend yield of stock i in year y, calculated as
				 the sum of the dividends during year y, divided by the end 
				 of year price (p. 39). Follows Brennan et al 1998 so dividends paid. ***/

		*File: input.dividends2: all permnos from 1960-2015, limited variables. 
			Downloaded from the CRSP distribution file
			(https://wrds-web.wharton.upenn.edu/wrds/ds/crsp/stock_a/dist.cfm?navId=129) 
			DISTCD (Distribution Code)
			DIVAMT (Dividend Cash Amount)
			PAYDT (Payment Date)
			FACPR (Factor to Adjust Price);

proc freq data=input.dividends2 nlevels; tables DISTCD; run;

data dividends;
	set input.dividends2;
	where DISTCD < 2000 and DIVAMT > 0 and PAYDT is not missing; 
	year=year(PAYDT); 
run;

proc sql;
create table work.dividends_pa as
select distinct permno, year, 
	sum(DIVAMT) as annual_dividend 
from work.dividends 
group by permno, year
;

/* ROBUSTNESS CHECK #3: get share price as at end of second to last month in that year, not last month.
Need: 
1) the year-permno obervations from ILLIQ_trimmed.
2) the share price as at November of each year */

data monthly_November_price;
	set input.CRSP_monthly_stock_all;
	keep permno date year prc nov_prc;
	where month(date)=11;
	year=year(date);
	Nov_prc=prc;
run;

/* check for duplicate observations and delete */
PROC FREQ data=monthly_November_price; TABLES permno*year / noprint out=keylist; RUN;
data keylist2; set keylist; where count ge 2; RUN;
proc sort data=monthly_November_price nodupkey; by permno year; run; 
data monthly_November_price;
	set monthly_November_price;
	drop date prc;
run;
Data ILLIQ_trimmed; merge ILLIQ_trimmed (in=a) monthly_November_price (in=b); by permno year; if a; run;
Data ILLIQ_trimmed; merge ILLIQ_trimmed (in=a) dividends_pa (in=b);	by permno year; if a; run;
data ILLIQ_trimmed;
	set ILLIQ_trimmed;
	if annual_dividend = . then DIVYLD = 0;
	else DIVYLD = annual_dividend/abs(Nov_prc); /************** ROBUSTNESS CHECK #3 *************/
	DIVYLD_PCT=DIVYLD*100;
run;

* Delete datasets no longer needed;
proc datasets library=work; 
	delete Delist_date Dividends R100 R100yr R100_bgn R100_end R100_prep; run;


	/*** BETA_SW: the slope coefficient, estimated by the Scholes and Williams 1977
				  method, of the market model R(pty)=a(py)+Beta(py)*RM(ty)+e(pty).
				  Sort stocks into size (market capitalisation) deciles each year.
				  Beta of stock i in year y = beta of the size decile to which that
                  stock belongs.***/

* For codes for each CRSP index, see: 
		   http://www.crsp.com/products/documentation/appendix-crsp-index-series
  The file input.Nyse_cap_portfolios_daily contains Daily NYSE market returns from 
	       Jan 1960-Dec 2014, including the returns on cap based NYSE portfolios 
	       (https://wrds-web.wharton.upenn.edu/wrds/query_forms/navigation.cfm?navId=124);
	
*Rank stocks each year y by size and sort into 10 portfolios
*Calculate R(pty): equal weighted average of stock returns in port p on day t in year y;

data work.beta_regression_a;
	set work.ILLIQ_trimmed;
	keep permno date year MKT_CAP;
run;
proc sort data=beta_regression_a; by year MKT_CAP; run;
proc rank data=work.beta_regression_a groups=10 out=work.beta_sort;
	var MKT_CAP;
	ranks rank_MKT_CAP;
	by year;
run;
data work.beta_sort; set work.beta_sort; MKT_CAP_decile=rank_MKT_CAP+1; run;

* Delete datasets no longer needed;
proc datasets library=work; delete beta_regression_a; run;

proc sort data=beta_sort out=beta_sort (keep=permno year MKT_CAP_decile); by permno year; run;

Data beta_regression_b; merge CRSP_daily_stock_NYSE (in=a) beta_sort (in=b); by permno year; if a; run;
data beta_regression_b; set beta_regression_b; 
	where MKT_CAP_decile is not missing; keep permno date MKT_CAP_decile RET; run;
proc sort data=beta_regression_b; by date permno; run;

proc sql;
create table work.beta_regression as
select distinct date, 
	count(RET) as NUM_STOCKS, 
	sum(RET)/count(RET) as PORTFOLIO_RET,
	sum(MKT_CAP_decile)/count(MKT_CAP_decile) as MKT_CAP_decile
from work.beta_regression_b 
group by date, MKT_CAP_decile
;
quit;
proc sort data=beta_regression; by date MKT_CAP_decile; run;

*Adjust market return file to include lag and lead values using proc expand then merge with portfolio return;

data work.Daily_mkt_return;
	set input.Nyse_cap_portfolios_daily;
	rename caldt=date;
run;
data work.Daily_mkt_return;
	set Daily_mkt_return;
	keep date EWRETD RM Year; 
	Year=year(date);
	RM=EWRETD; *EWRETD = equal weighted market return including dividends;
run;

proc expand data=Daily_mkt_return out=Rm method = none;
	id date;
	convert Rm = Rm_lag1 / transformout=(lag 1);
	convert Rm;
	convert Rm = Rm_lead1 / transformout=( lead 1 );
run;

Data beta_regression; merge beta_regression (in=a) Rm (in=b); by date; if a; run;

* Rm;
Data beta_regression_S1; set beta_regression; where mkt_cap_decile=1; run;
Data beta_regression_S2; set beta_regression; where mkt_cap_decile=2; run;
Data beta_regression_S3; set beta_regression; where mkt_cap_decile=3; run;
Data beta_regression_S4; set beta_regression; where mkt_cap_decile=4; run;
Data beta_regression_S5; set beta_regression; where mkt_cap_decile=5; run;
Data beta_regression_S6; set beta_regression; where mkt_cap_decile=6; run;
Data beta_regression_S7; set beta_regression; where mkt_cap_decile=7; run;
Data beta_regression_S8; set beta_regression; where mkt_cap_decile=8; run;
Data beta_regression_S9; set beta_regression; where mkt_cap_decile=9; run;
Data beta_regression_S10; set beta_regression; where mkt_cap_decile=10; run;

Proc reg data=beta_regression_S1 outest=beta_rm_S1 noprint; model Portfolio_ret=rm; by year; run;
Proc reg data=beta_regression_S2 outest=beta_rm_S2 noprint; model Portfolio_ret=rm; by year; run;
Proc reg data=beta_regression_S3 outest=beta_rm_S3 noprint; model Portfolio_ret=rm; by year; run;
Proc reg data=beta_regression_S4 outest=beta_rm_S4 noprint; model Portfolio_ret=rm; by year; run;
Proc reg data=beta_regression_S5 outest=beta_rm_S5 noprint; model Portfolio_ret=rm; by year; run;
Proc reg data=beta_regression_S6 outest=beta_rm_S6 noprint; model Portfolio_ret=rm; by year; run;
Proc reg data=beta_regression_S7 outest=beta_rm_S7 noprint; model Portfolio_ret=rm; by year; run;
Proc reg data=beta_regression_S8 outest=beta_rm_S8 noprint; model Portfolio_ret=rm; by year; run;
Proc reg data=beta_regression_S9 outest=beta_rm_S9 noprint; model Portfolio_ret=rm; by year; run;
Proc reg data=beta_regression_S10 outest=beta_rm_S10 noprint; model Portfolio_ret=rm; by year; run;

data beta_rm_S1; set beta_rm_S1; mkt_cap_decile=1; run; 
data beta_rm_S2; set beta_rm_S2; mkt_cap_decile=2; run;
data beta_rm_S3; set beta_rm_S3; mkt_cap_decile=3; run;
data beta_rm_S4; set beta_rm_S4; mkt_cap_decile=4; run;
data beta_rm_S5; set beta_rm_S5; mkt_cap_decile=5; run;
data beta_rm_S6; set beta_rm_S6; mkt_cap_decile=6; run;
data beta_rm_S7; set beta_rm_S7; mkt_cap_decile=7; run;
data beta_rm_S8; set beta_rm_S8; mkt_cap_decile=8; run;
data beta_rm_S9; set beta_rm_S9; mkt_cap_decile=9; run;
data beta_rm_S10; set beta_rm_S10; mkt_cap_decile=10; run;

data beta_rm; 
	set beta_rm_S1 beta_rm_S2 beta_rm_S3 beta_rm_S4 beta_rm_S5
	    beta_rm_S6 beta_rm_S7 beta_rm_S8 beta_rm_S9 beta_rm_S10;
	beta=Rm;
run; 

* Rm_lag;
Data beta_regression_S1; set beta_regression; where mkt_cap_decile=1; run;
Data beta_regression_S2; set beta_regression; where mkt_cap_decile=2; run;
Data beta_regression_S3; set beta_regression; where mkt_cap_decile=3; run;
Data beta_regression_S4; set beta_regression; where mkt_cap_decile=4; run;
Data beta_regression_S5; set beta_regression; where mkt_cap_decile=5; run;
Data beta_regression_S6; set beta_regression; where mkt_cap_decile=6; run;
Data beta_regression_S7; set beta_regression; where mkt_cap_decile=7; run;
Data beta_regression_S8; set beta_regression; where mkt_cap_decile=8; run;
Data beta_regression_S9; set beta_regression; where mkt_cap_decile=9; run;
Data beta_regression_S10; set beta_regression; where mkt_cap_decile=10; run;

Proc reg data=beta_regression_S1 outest=beta_rm_lag_S1 noprint; model Portfolio_ret=Rm_lag1; by year; run;
Proc reg data=beta_regression_S2 outest=beta_rm_lag_S2 noprint; model Portfolio_ret=Rm_lag1; by year; run;
Proc reg data=beta_regression_S3 outest=beta_rm_lag_S3 noprint; model Portfolio_ret=Rm_lag1; by year; run;
Proc reg data=beta_regression_S4 outest=beta_rm_lag_S4 noprint; model Portfolio_ret=Rm_lag1; by year; run;
Proc reg data=beta_regression_S5 outest=beta_rm_lag_S5 noprint; model Portfolio_ret=Rm_lag1; by year; run;
Proc reg data=beta_regression_S6 outest=beta_rm_lag_S6 noprint; model Portfolio_ret=Rm_lag1; by year; run;
Proc reg data=beta_regression_S7 outest=beta_rm_lag_S7 noprint; model Portfolio_ret=Rm_lag1; by year; run;
Proc reg data=beta_regression_S8 outest=beta_rm_lag_S8 noprint; model Portfolio_ret=Rm_lag1; by year; run;
Proc reg data=beta_regression_S9 outest=beta_rm_lag_S9 noprint; model Portfolio_ret=Rm_lag1; by year; run;
Proc reg data=beta_regression_S10 outest=beta_rm_lag_S10 noprint; model Portfolio_ret=Rm_lag1; by year; run;

data beta_rm_lag_S1; set beta_rm_lag_S1; mkt_cap_decile=1; run; 
data beta_rm_lag_S2; set beta_rm_lag_S2; mkt_cap_decile=2; run;
data beta_rm_lag_S3; set beta_rm_lag_S3; mkt_cap_decile=3; run;
data beta_rm_lag_S4; set beta_rm_lag_S4; mkt_cap_decile=4; run;
data beta_rm_lag_S5; set beta_rm_lag_S5; mkt_cap_decile=5; run;
data beta_rm_lag_S6; set beta_rm_lag_S6; mkt_cap_decile=6; run;
data beta_rm_lag_S7; set beta_rm_lag_S7; mkt_cap_decile=7; run;
data beta_rm_lag_S8; set beta_rm_lag_S8; mkt_cap_decile=8; run;
data beta_rm_lag_S9; set beta_rm_lag_S9; mkt_cap_decile=9; run;
data beta_rm_lag_S10; set beta_rm_lag_S10; mkt_cap_decile=10; run;

data beta_rm_lag; 
	set beta_rm_lag_S1 beta_rm_lag_S2 beta_rm_lag_S3 beta_rm_lag_S4 beta_rm_lag_S5
	    beta_rm_lag_S6 beta_rm_lag_S7 beta_rm_lag_S8 beta_rm_lag_S9 beta_rm_lag_S10;
	beta_lag=Rm_lag1;
run; 

* Rm_lead;
Data beta_regression_S1; set beta_regression; where mkt_cap_decile=1; run;
Data beta_regression_S2; set beta_regression; where mkt_cap_decile=2; run;
Data beta_regression_S3; set beta_regression; where mkt_cap_decile=3; run;
Data beta_regression_S4; set beta_regression; where mkt_cap_decile=4; run;
Data beta_regression_S5; set beta_regression; where mkt_cap_decile=5; run;
Data beta_regression_S6; set beta_regression; where mkt_cap_decile=6; run;
Data beta_regression_S7; set beta_regression; where mkt_cap_decile=7; run;
Data beta_regression_S8; set beta_regression; where mkt_cap_decile=8; run;
Data beta_regression_S9; set beta_regression; where mkt_cap_decile=9; run;
Data beta_regression_S10; set beta_regression; where mkt_cap_decile=10; run;

Proc reg data=beta_regression_S1 outest=beta_rm_lead_S1 noprint; model Portfolio_ret=Rm_lead1; by year; run;
Proc reg data=beta_regression_S2 outest=beta_rm_lead_S2 noprint; model Portfolio_ret=Rm_lead1; by year; run;
Proc reg data=beta_regression_S3 outest=beta_rm_lead_S3 noprint; model Portfolio_ret=Rm_lead1; by year; run;
Proc reg data=beta_regression_S4 outest=beta_rm_lead_S4 noprint; model Portfolio_ret=Rm_lead1; by year; run;
Proc reg data=beta_regression_S5 outest=beta_rm_lead_S5 noprint; model Portfolio_ret=Rm_lead1; by year; run;
Proc reg data=beta_regression_S6 outest=beta_rm_lead_S6 noprint; model Portfolio_ret=Rm_lead1; by year; run;
Proc reg data=beta_regression_S7 outest=beta_rm_lead_S7 noprint; model Portfolio_ret=Rm_lead1; by year; run;
Proc reg data=beta_regression_S8 outest=beta_rm_lead_S8 noprint; model Portfolio_ret=Rm_lead1; by year; run;
Proc reg data=beta_regression_S9 outest=beta_rm_lead_S9 noprint; model Portfolio_ret=Rm_lead1; by year; run;
Proc reg data=beta_regression_S10 outest=beta_rm_lead_S10 noprint; model Portfolio_ret=Rm_lead1; by year; run;

data beta_rm_lead_S1; set beta_rm_lead_S1; mkt_cap_decile=1; run; 
data beta_rm_lead_S2; set beta_rm_lead_S2; mkt_cap_decile=2; run;
data beta_rm_lead_S3; set beta_rm_lead_S3; mkt_cap_decile=3; run;
data beta_rm_lead_S4; set beta_rm_lead_S4; mkt_cap_decile=4; run;
data beta_rm_lead_S5; set beta_rm_lead_S5; mkt_cap_decile=5; run;
data beta_rm_lead_S6; set beta_rm_lead_S6; mkt_cap_decile=6; run;
data beta_rm_lead_S7; set beta_rm_lead_S7; mkt_cap_decile=7; run;
data beta_rm_lead_S8; set beta_rm_lead_S8; mkt_cap_decile=8; run;
data beta_rm_lead_S9; set beta_rm_lead_S9; mkt_cap_decile=9; run;
data beta_rm_lead_S10; set beta_rm_lead_S10; mkt_cap_decile=10; run;

data beta_rm_lead; 
	set beta_rm_lead_S1 beta_rm_lead_S2 beta_rm_lead_S3 beta_rm_lead_S4 beta_rm_lead_S5
	    beta_rm_lead_S6 beta_rm_lead_S7 beta_rm_lead_S8 beta_rm_lead_S9 beta_rm_lead_S10;
	beta_lead=Rm_lead1;
run; 

* Merge the beta files together, keeping only needed variables;
proc sort data=beta_rm out=beta_rm (keep=year mkt_cap_decile beta); by year mkt_cap_decile; run;
proc sort data=beta_rm_lag out=beta_rm_lag (keep=year mkt_cap_decile beta_lag); by year mkt_cap_decile; run;
proc sort data=beta_rm_lead out=beta_rm_lead (keep=year mkt_cap_decile beta_lead); by year mkt_cap_decile; run;

Data beta; 
	merge beta_rm (in=a) beta_rm_lag (in=b) beta_rm_lead (in=c); by year mkt_cap_decile; if a; run;

* calculate autocorrelation: nValue1;
proc reg data = Rm;
  model Rm = /dw;
  by year;
 ods output dwstatistic = auto_corr 
	(where=(label1="1st Order Autocorrelation")) ;
run;
quit;
data auto_corr; set auto_corr; keep year autocorr_rm; autocorr_rm=nvalue1; run;

* merge aurocorrelation in with betas;
Data beta; 
	merge beta (in=a) auto_corr (in=b); by year; if a; run;

* calculate statistic;
data beta; set beta; beta_sw=(beta_lag + beta + beta_lead)/(1 + (2*autocorr_rm)); run;

* merge into ILLIQ_trimmed main data set by year and size decile allocation;
proc sort data=beta_sort; by year MKT_CAP_decile; run;
Data beta_sw; merge beta_sort (in=a) beta (in=b); by year MKT_CAP_decile; if a; run;

proc sort data=beta_sw; by permno year; run;
Data ILLIQ_trimmed; merge ILLIQ_trimmed (in=a) beta_sw (in=b);	by permno year; if a; run;

* Delete datasets no longer needed;
proc datasets library=work; 
	delete Beta_regression_s1 Beta_regression_s2 Beta_regression_s3 Beta_regression_s4 Beta_regression_s5
		   Beta_regression_s6 Beta_regression_s7 Beta_regression_s8 Beta_regression_s9 Beta_regression_s10
		   Beta_rm Beta_rm_lag Beta_rm_lead 
		   beta_rm_S1 beta_rm_S2 beta_rm_S3 beta_rm_S4 beta_rm_S5
	       beta_rm_S6 beta_rm_S7 beta_rm_S8 beta_rm_S9 beta_rm_S10
		   beta_rm_lag_S1 beta_rm_lag_S2 beta_rm_lag_S3 beta_rm_lag_S4 beta_rm_lag_S5
	       beta_rm_lag_S6 beta_rm_lag_S7 beta_rm_lag_S8 beta_rm_lag_S9 beta_rm_lag_S10
		   beta_rm_lead_S1 beta_rm_lead_S2 beta_rm_lead_S3 beta_rm_lead_S4 beta_rm_lead_S5
	       beta_rm_lead_S6 beta_rm_lead_S7 beta_rm_lead_S8 beta_rm_lead_S9 beta_rm_lead_S10; 
run;

proc means data=work.ILLIQ_trimmed n nmiss mean min max; run;









/*==========================================================================================*/
/*==========================================================================================*/

					/************** COMPARATIVE STATISTICS **************/

/*==========================================================================================*/
/*==========================================================================================*/

proc sql;
create table work.stock_year_obs as
select distinct year, 
	count(permno) as num_stocks
from work.ILLIQ_trimmed 
group by year
;

data work.stock_year_obs;
	set work.stock_year_obs;
	where year >= 1963 and year <= 2014; /* restrain sample period */
run;

/* ------------------ COMPARATIVE STATISTIC 1: # STOCKS P.A. (AMIHUD P.37) ------------------ */

proc means data=work.stock_year_obs n nmiss min max;
	var num_stocks;
run;

/* ----------------- COMPARATIVE STATISTIC 2: MEAN ILLIQ P.A. (AMIHUD P.38) ----------------- */

proc sql;
create table work.illiq_statistics as
select distinct year, 
	count(permno) as num_stocks,
	mean(ILLIQ_1000000) as pa_mean_ILLIQ,
	std(ILLIQ_1000000) as pa_stdev_ILLIQ,
	mean(ILLIQMA) as pa_mean_ILLIQMA,
	std(ILLIQMA) as pa_stdev_ILLIQMA,
	mean(MKT_CAP_MILLIONS) as pa_mean_size,
	std(MKT_CAP_MILLIONS) as pa_stdev_size,
	mean(STDEV_RET_100) as pa_mean_stdev_ret,
	std(STDEV_RET_100) as pa_stdev_stdev_ret,
	mean(DIVYLD_PCT) as pa_mean_DIVYLD_PCT,
	std(DIVYLD_PCT) as pa_stdev_DIVYLD_PCT
from work.ILLIQ_trimmed 
group by year
;

data work.illiq_statistics;
	set work.illiq_statistics;
	where year >= 1963 and year <= 2014;
run;

proc means data=work.illiq_statistics n nmiss min max mean median;
	var pa_mean_ILLIQ pa_stdev_ILLIQ pa_mean_size pa_stdev_size pa_mean_stdev_ret pa_stdev_stdev_ret 
		pa_mean_DIVYLD_PCT pa_stdev_DIVYLD_PCT;
run;

	/* ----- correlation calculation: ILLIQ and SIZE ----- */
proc sort data = ILLIQ_trimmed out = sort_correl; by year permno; run;
proc corr data=sort_correl outp=correl noprint; by year; var ILLIQ ln_size_mill; run;

data work.correl2; 
	set work.correl;
	where _TYPE_="CORR" and _NAME_="ILLIQ";
run;

data work.correl3;
	set work.correl2;
	where year >= 1963 and year <= 2014;
run;

proc means data=work.correl3 n nmiss min max mean median;
	var ln_size_mill;
run;

	/* ----- correlation calculation: ILLIQ & SDRET ----- */
proc sort data = ILLIQ_trimmed out = sort_correlb; by year permno; run;
proc corr data=sort_correlb outp=correlb noprint; by year; var ILLIQ STDEV_RET_100; run;

data work.correlb2; 
	set work.correlb;
	where _TYPE_="CORR" and _NAME_="ILLIQ"; 
run;

data work.correlb3; 
	set work.correlb2;
	where year >= 1963 and year <= 2014;
run;

proc means data=work.correlb3 n nmiss min max mean median;
	var STDEV_RET_100;
run;

* Delete datasets no longer needed;
proc datasets library=work; delete correl; run;
















/*==========================================================================================*/
/*==========================================================================================*/

			/************** Monthly returns - delisting adjustment **************/

/*==========================================================================================*/
/*==========================================================================================*/

data monthly_delist_return;
	set input.CRSP_monthly_stock_all;
	keep permno date ret dlstcd dlpdt dlret;
run;

data delist_all;
	set monthly_delist_return;
	where dlstcd is not missing;
run;

/* STEPS 1 and 2:
Find observations with no value for RET. If there is a DLRET use this. If no
DLRET and code is in 500s as listed above then adj_DLRET = -0.3. Otherwise 
set DLRET to missing*/

data delist_1and2;
	set monthly_delist_return;
	where dlstcd is not missing and ret is missing;
run;
proc freq data=delist_1and2 NLEVELS; tables dlstcd; run;

data delist_1and2;
	set delist_1and2;
	if missing(dlret) then miss_code=1;
	else miss_code=0;
run;

data delist_1and2;
	set delist_1and2;
	if miss_code = 0 then adj_DLRET = dlret;
	else if dlstcd = 500 
		or dlstcd = 520
		or dlstcd = 580
		or dlstcd = 574
		or dlstcd = 584
		or (dlstcd >= 551 and dlstcd <= 573) then adj_DLRET = -0.3;
	else adj_DLRET = .; 
run;

data check_1and2; set delist_1and2; where miss_code = 1; run;

/* STEP 3:
Find observations with value for RET but no value for DLRET. 
If code is not in 500s as listed above then use adj_DLRET=RET
Otherwise: if code is in the list above (1.26% of observations),
adjust for delisting return = -30% by adj_DLRET = ((1+RET)*(1-0.3))-1;
*/

data delist_3;
	set monthly_delist_return;
	where dlstcd is not missing and RET is not missing and dlret is missing;
run;
proc freq data=delist_3 NLEVELS; tables dlstcd; run;

data delist_3;
	set delist_3;
	if dlstcd = 500 
		or dlstcd = 520
		or dlstcd = 580
		or dlstcd = 574
		or dlstcd = 584
		or (dlstcd >= 551 and dlstcd <= 573) then miss_code = 1;
	else miss_code=0;
run;
proc freq data=delist_3 NLEVELS; tables miss_code; run;

data delist_3;
	set delist_3;
	if miss_code = 0 then adj_DLRET = ret;
	else if miss_code = 1 then adj_DLRET = ((1+RET)*(1-0.3))-1; 
		/******** in this case adjust for delisting return of -30% **********/
run;

data check_3; set delist_3; where miss_code = 1; run;

/* STEP 4:
If BOTH the RET and DLRET are available, this occurs when Date of
delisting payment is after the last monthly RET. 
Adjust for delisting return by adj_DLRET = ((1+RET)*(1+DLRET))-1;
Merge with delist file to check that date of RET = Date of delisting.
*/

data delist_4;
	set monthly_delist_return;
	where dlstcd is not missing and RET is not missing and dlret is not missing;
run;
proc freq data=delist_4 NLEVELS; tables dlstcd; run;

data delist_4;
	set delist_4;
	adj_DLRET = ((1+RET)*(1+DLRET))-1; 
		/******** in this case adjust for delisting return provided **********/
run;

/* check: can see in all cases date=delist date */
data delist_date; set input.delist; keep permno DLSTDT; run;
Data delist_4; merge delist_4 (in=a) delist_date (in=b);	by permno; if a; run;
data check4; set delist_4; where date ne DLSTDT; run;

/* stack the datasets, sort, keep only needed variables and delete duplicated for merge */
Data delist_returns; set delist_1and2 delist_3 delist_4; run;
proc sort data=delist_returns (keep=permno date adj_DLRET); by permno date; run;
proc sort data=delist_returns nodupkey; by permno date; run;

/* merge file in to main dataset using permno and date, keep only needed variables */
Data CRSP_monthly_stock_delist_adj; 	
	merge input.CRSP_monthly_stock_all (in=a) delist_returns (in=b); 
	by permno date; if a; run;

proc contents data=CRSP_monthly_stock_delist_adj; run;

data CRSP_monthly_stock_delist_adj; 
	set CRSP_monthly_stock_delist_adj; 
	keep permno date SHRCD EXCHCD NAMEENDT PRIMEXCH TRDSTAT RET 
		 dlstcd dlpdt dlret adj_DLRET adj_ret adj_ret_100 adj_ret_100_pa;
	if missing(adj_DLRET) then adj_ret=ret;
	else adj_ret=adj_DLRET;
	adj_ret_100=adj_ret*100;
	adj_ret_100_pa=adj_ret*100*12;
run;

* Delete datasets no longer needed;
proc datasets library=work; 
	delete check_1and2 check_3 check4 Delist_1and2 Delist_3 Delist_4 monthly_delist_return; run;














/*==========================================================================================*/
/*==========================================================================================*/

			/************** CROSS-SECTIONAL ESTIMATIONS **************/

/*==========================================================================================*/
/*==========================================================================================*/

* Clean monthly return file, create lagged variable for stock characteristics and merge;

data CRSP_monthly_stock_delist_adj; 
	set CRSP_monthly_stock_delist_adj; 
	where adj_ret is not missing;
	year=year(date);
run;

proc sort data=Illiq_trimmed; by permno year; run;

* Lag the variables;
proc sort data=Illiq_trimmed out=final_year; 
	by permno descending year;
run;
proc sort data=final_year nodupkey; by permno; run;
data final_year; set final_year; keep permno /* year */ next_year;
	next_year = year+1;
run;
data final_year; set final_year;
	rename next_year = year;
run;
data Illiq_trimmed; set Illiq_trimmed final_year; run;
proc sort data=Illiq_trimmed; by permno year; run;
data Illiq_trimmed_lag;
	set Illiq_trimmed;
	BY PERMNO YEAR;
	drop Sum_Abs_ret_p_dollar_vol DLSTDT Date PRC AILLIQ annual_dividend 
		 MKT_CAP_decile beta_lag beta_lead autocorr_rm;
	Year_lag = Year-1;
	EOY_Date = Date;
	Format EOY_Date date9.;
	Format EOY_Date_lag date9.;
	* Create lagged variable;
	BETA_SW_lag=lag(beta_sw);
	ILLIQMA_lag=lag(ILLIQMA*100); 
	R100_lag=lag(R100*100); 
	R100YR_lag=lag(R100YR*100);
	SIZE_lag=lag(ln_size_mill); 
	SDRET_lag=lag(STDEV_RET*100);
	DIVYLD_lag=lag(DIVYLD_PCT); 
	EOY_Date_lag = lag(Date);
	* remove lag where first observation;
	IF FIRST.PERMNO THEN DO;
		BETA_SW_lag = .;
		ILLIQMA_lag = .;
		R100_lag = .;
		R100YR_lag = .;
		SIZE_lag = .;
		SDRET_lag = .;
		DIVYLD_lag = .;
		EOY_Date_lag = .;
	END;
	* remove lag where previous year is missing (i.e. skipped year);
	IF LAG(YEAR) NE (Year-1) THEN DO;
		BETA_SW_lag = .;
		ILLIQMA_lag = .;
		R100_lag = .;
		R100YR_lag = .;
		SIZE_lag = .;
		SDRET_lag = .;
		DIVYLD_lag = .;
		EOY_Date_lag = .;
	END;
run;
	
Data CRSP_monthly_stock_delist_adj; 	
	merge CRSP_monthly_stock_delist_adj (in=a) Illiq_trimmed_lag (in=b); 
	by permno year; if a; run;

* only include stocks with data for characteristics;
data CRSP_monthly_stock_delist_adj; set CRSP_monthly_stock_delist_adj; 
	where ILLIQMA_lag is not missing; run;

proc sort data=CRSP_monthly_stock_delist_adj nodupkey; by permno date; run;











/************************************************************************************/
	   	  * DSV Table 4, Panel A: FAMA-MACBETH Cross-sectional regressions;
/************************************************************************************/

data CRSP_monthly_stock_delist_adj;
	set CRSP_monthly_stock_delist_adj;
	where year >=1964 and year <=2015;
run;

* Cross-sectional regression;

* Table 4, Panel A column 1;
proc sort data=CRSP_monthly_stock_delist_adj out=cross_sectional_reg_c1; by date permno; run;
proc reg data=cross_sectional_reg_c1 outest=cross_sectional_reg2c1 noprint;
 by date;
 model Adj_ret_100 = BETA_SW_lag ILLIQMA_lag R100_lag R100YR_lag; 
run;
proc means data=cross_sectional_reg2c1 n nmiss mean std t probt;
 var intercept BETA_SW_lag ILLIQMA_lag R100_lag R100YR_lag; 
run;


* Table 4, Panel A column 2;
proc sort data=CRSP_monthly_stock_delist_adj out=cross_sectional_reg_c2; by date permno;
	where month(date) > 1; 
run;
proc reg data=cross_sectional_reg_c2 outest=cross_sectional_reg2c2 noprint;
 by date;
 model Adj_ret_100 = BETA_SW_lag ILLIQMA_lag R100_lag R100YR_lag; 
run;
proc means data=cross_sectional_reg2c2 n nmiss mean std t probt;
 var intercept BETA_SW_lag ILLIQMA_lag R100_lag R100YR_lag; 
run;


* Table 4, Panel A column 3;
proc sort data=CRSP_monthly_stock_delist_adj out=cross_sectional_reg_c5; by date permno; run;
proc reg data=cross_sectional_reg_c5 outest=cross_sectional_reg2c5 noprint;
 by date;
 model Adj_ret_100 = BETA_SW_lag ILLIQMA_lag R100_lag R100YR_lag SIZE_lag SDRET_lag DIVYLD_lag; 
run;
proc means data=cross_sectional_reg2c5 n nmiss mean std t probt;
 var intercept BETA_SW_lag ILLIQMA_lag R100_lag R100YR_lag SIZE_lag SDRET_lag DIVYLD_lag;
run;


* Table 4, Panel A column 4;
proc sort data=CRSP_monthly_stock_delist_adj out=cross_sectional_reg_c6; by date permno;
	where month(date) > 1; 
run;
proc reg data=cross_sectional_reg_c6 outest=cross_sectional_reg2c6 noprint;
 by date;
 model Adj_ret_100 = BETA_SW_lag ILLIQMA_lag R100_lag R100YR_lag SIZE_lag SDRET_lag DIVYLD_lag; 
run;
proc means data=cross_sectional_reg2c6 n nmiss mean std t probt;
 var intercept BETA_SW_lag ILLIQMA_lag R100_lag R100YR_lag SIZE_lag SDRET_lag DIVYLD_lag;
run;


* Delete datasets no longer needed;
proc datasets library=work; 
	delete 
Cross_sectional_reg_c1
Cross_sectional_reg_c2
Cross_sectional_reg_c5
Cross_sectional_reg_c6; 
run;

* check statistics on p 40;
data Table2_column1_stat;
	set Cross_sectional_reg2c1;
	keep Date ILLIQMA_lag ILLIQMA_lag_100 positive;
	ILLIQMA_lag_100=ILLIQMA_lag*100;
	If ILLIQMA_lag > 0 then positive=1;
	else positive=0;
run;

proc means data=Table2_column1_stat n nmiss mean median std t;
 var ILLIQMA_lag_100;
run;

proc sql;
create table work.Table2_column1_stat2 as
	select
	sum(positive) as num_positive,
	count(ILLIQMA_lag) as num_total
from work.Table2_column1_stat 
;

proc reg data = Table2_column1_stat;
  model ILLIQMA_lag_100 = /dw;
 ods output dwstatistic = Table2_column1_stat3 
	(where=(label1="1st Order Autocorrelation")) ;
run;
quit;

















/*==========================================================================================*/
/*==========================================================================================*/

			 /************** TIME_SERIES ESTIMATIONS - ANNUAL **************/

/*==========================================================================================*/
/*==========================================================================================*/

proc sort data=CRSP_daily_stock_NYSE; by permno year; run;

/*===================================================================*/
/* create dollar volume (DOLLAR_VOL) variable and absolute return/dollar volume variable 
for ILLIQ calculation, adjusted for inflation;  */

proc sql;
create table work.ILLIQ as
select distinct permno, year, 
	count(Abs_ret_per_dol_vol_INF) as num_D, 
	sum(Abs_ret_per_dol_vol_INF) as Sum_Abs_ret_p_dollar_vol
from work.CRSP_daily_stock_NYSE 
group by permno, year
;
/*===================================================================*/

data work.ILLIQ; set work.ILLIQ; ILLIQ=Sum_Abs_ret_p_dollar_vol/num_D; run;

data work.ILLIQ_requirements;
	set input.CRSP_daily_stock_NYSE;
	keep permno year date PRC SHROUT SHRCD EXCHCD PRIMEXCH year MKT_CAP;
	Year=year(DATE);
	MKT_CAP=abs(prc)*SHROUT;
run;

proc sort data=ILLIQ_requirements;
	by permno year descending date;
run;

proc sort data=ILLIQ_requirements nodupkey;
	by permno year;
run;

data work.ILLIQ_v2;
	merge 	work.ILLIQ (in=a)
			work.ILLIQ_requirements (in=b);
	by permno year;
	if a;
run;
proc freq data=ILLIQ_v2 nlevels; TABLES EXCHCD PRIMEXCH SHRCD; RUN;

data work.ILLIQ_TS_tests;
	set work.ILLIQ_v2;
	where SHRCD < 20; /* RESTRICT SHARE CODE */
	keep permno year ILLIQ; 
run;

proc sort data=ILLIQ_TS_TESTS; by year ILLIQ; run;

* Winsorise upper 1% tail of data;

%macro Winsorize_Truncate(dsetin = work.ILLIQ_TS_TESTS, 
                          dsetout = work.ILLIQ_trimmed_TS, 
                          byvar = year, 
                          vars = ILLIQ, 
                          type = W, /* T for truncate, vs W for winsorize */
                          pctl = 0 99);

    %if &dsetout = %then %let dsetout = &dsetin;
    
    %let varL=;
    %let varH=;
    %let xn=1;

    %do %until (%scan(&vars,&xn)= );
        %let token = %scan(&vars,&xn);
        %let varL = &varL &token.L;
        %let varH = &varH &token.H;
        %let xn = %EVAL(&xn + 1);
    %end;

    %let xn = %eval(&xn-1);

    data xtemp;
        set &dsetin;

    %let dropvar = ;
    %if &byvar = none %then %do;
        data xtemp;
            set xtemp;
            xbyvar = 1;

        %let byvar = xbyvar;
        %let dropvar = xbyvar;
    %end;

    proc sort data = xtemp;
        by &byvar;

    /*compute percentage cutoff values*/
    proc univariate data = xtemp noprint;
        by &byvar;
        var &vars;
        output out = xtemp_pctl PCTLPTS = &pctl PCTLPRE = &vars PCTLNAME = L H;

    data &dsetout;
        merge xtemp xtemp_pctl; /*merge percentage cutoff values into main dataset*/
        by &byvar;
        array trimvars{&xn} &vars;
        array trimvarl{&xn} &varL;
        array trimvarh{&xn} &varH;

        do xi = 1 to dim(trimvars);
            /*winsorize variables*/
            %if &type = W %then %do;
                if trimvars{xi} ne . then do;
                    if (trimvars{xi} < trimvarl{xi}) then trimvars{xi} = trimvarl{xi};
                    if (trimvars{xi} > trimvarh{xi}) then trimvars{xi} = trimvarh{xi};
                end;
            %end;
            /*truncate variables*/
            %else %do;
                if trimvars{xi} ne . then do;
                    /*insert .T code if value is truncated*/
                    if (trimvars{xi} < trimvarl{xi}) then trimvars{xi} = .T;
                    if (trimvars{xi} > trimvarh{xi}) then trimvars{xi} = .T;
                end;
            %end;
        end;
        drop &varL &varH &dropvar xi;

    /*delete temporary datasets created during macro execution*/
    proc datasets library=work nolist;
        delete xtemp xtemp_pctl; quit; run;

%mend Winsorize_Truncate;

/*invoke macro to winsorize */

%Winsorize_Truncate(dsetin= work.ILLIQ_TS_TESTS, dsetout= work.ILLIQ_trimmed_TS, byvar=year, vars=ILLIQ, 
	type = W, pctl=0 99);
run;

data work.ILLIQ_trimmed_TS;
	set work.ILLIQ_trimmed_TS;
	where ILLIQ is not missing; 
	ILLIQ_1000000=ILLIQ*1000000; /* Amihud used ILLIQ*10^6 (p. 37)*/
run;

*AILLIQ = average ILLIQ across stocks in year y;
proc sort data=ILLIQ_trimmed_TS; by year permno; run;
proc sql;
create table work.AILLIQ_TS as
select distinct year, 
	sum(ILLIQ_1000000)/count(ILLIQ_1000000) as AILLIQ_TS, /* assume the transformation applies*/
	count(ILLIQ_1000000) as num_stocks
from work.ILLIQ_trimmed_TS 
group by year
;

data work.AILLIQ_TS;
	set work.AILLIQ_TS;
	ln_AILLIQ = log(AILLIQ_TS);
run;

*time series properties of the new ALLIQ measure;

title "Plot of AILLIQ";
   proc gplot data=AILLIQ_TS;
      symbol i=spline v=circle h=2;
      plot AILLIQ_TS * year;
   run;

* STEP 1: PERFORM REGRESSION IN EQ. (7): lnAILLIQy = c0 + c1 lnAILLIQy-1 + vy;
* add observation for the final year before lag;

data AILLIQ_TS_REG;
   set AILLIQ_TS;
   Year_lag = Year-1;
   AILLIQ_TS_lag = lag1( AILLIQ_TS );
   ln_AILLIQ_lag = lag1( ln_AILLIQ );
run;

data AILLIQ_TS_REG;
   set AILLIQ_TS_REG;
   where year >= 1964 and year <= 2015;
run;

data AILLIQ_TS_REG; set AILLIQ_TS_REG; 
	where ln_AILLIQ is not missing and ln_AILLIQ_lag is not missing; run;

proc model data=AILLIQ_TS_REG;
         parms b0 b1;
         ln_AILLIQ=b0 + b1*ln_AILLIQ_lag;
         fit ln_AILLIQ / DW; /* Durbin-Watson statistic */
         run;
         quit;

/************** ROBUSTNESS CHECK #4 *************/
/*Get Durbin h statistic*/
proc autoreg data=AILLIQ_TS_REG;
   model ln_AILLIQ = ln_AILLIQ_lag / lagdep=ln_AILLIQ_lag;
run;

*save needed variables to dataset and rename;
proc reg data=AILLIQ_TS_REG outest=Reg_Eq_7 edf;
	model ln_AILLIQ = ln_AILLIQ_lag; 
run;
quit;

data Reg_Eq_7_coeff (keep= C_0 C_1 T); set Reg_Eq_7; 
	label Intercept=' '; /* remove intercept label */
	rename Intercept=C_0;
	rename ln_AILLIQ_lag=C_1;
	T=_P_+_EDF_; 
	/* Need T for Kendall parameter adjustment. 
	   T (# of observations) = _P_ (# of parameters estimated, including the intercept) 
	   + _EDF_ (the error degrees of freedom)*/
run;


* STEP 2: ADJUST THE COEFFICIENTS FOR BIAS ACCORDING TO KENDALL (1954):
	c_1_adj = c_1 + ((1+3*C_1)/T);

data Reg_Eq_7_coeff; set Reg_Eq_7_coeff; 
	C_1_adj = C_1 + ((1+3*C_1)/T);
run;

*C_0 adjustment: c_0_adj = av(ln_AILLIQ) - c_1_adj*av(ln_AILLIQ_lag);

proc sql;
create table work.C_0_adj as
select 
	sum(ln_AILLIQ)/count(ln_AILLIQ) as av_ln_AILLIQ,
	sum(ln_AILLIQ_lag)/count(ln_AILLIQ_lag) as av_ln_AILLIQ_lag
from work.AILLIQ_TS_REG 
;

data Reg_Eq_7_coeff;
	merge Reg_Eq_7_coeff C_0_adj;
	C_0_adj = av_ln_AILLIQ - (c_1_adj*av_ln_AILLIQ_lag);
run;

*STEP 3: CALCULATE THE ADJUSTED RESIDUAL THROUGH MERGING BACK INTO YEARLY DATASET;

data AILLIQ_TS_REG; SET AILLIQ_TS_REG; Join = "Join"; run;
data Reg_Eq_7_coeff; SET Reg_Eq_7_coeff; Join = "Join"; run;

data AILLIQ_TS_REG;
	merge AILLIQ_TS_REG (in = a) Reg_Eq_7_coeff (in = b);
	by Join; if a;
run;

data AILLIQ_TS_REG; set AILLIQ_TS_REG;
	v_y = ln_AILLIQ - C_0 - (C_1*ln_AILLIQ_lag);
	v_y_adj = ln_AILLIQ - C_0_adj - (C_1_adj*ln_AILLIQ_lag);
run;

*STEP 4: MERGE IN WITH VARIABLES FOR REGRESSION OF EQ. 10:

 - Equally-weighted NYSE market return
 - 1 year T-bill yield as of BEGINNING of the year (source: Federal reserve bank)
 - annual return on CRSP size based portfolios (deciles)
   	Note: only value-weighted size deciles abailable. Use the CRSP Stock File Capitalization set,
   which puts the largest firms in Decile 10, consistent with Amihud p. 47;

* input.Nyse_cap_portfolios_annual contains annual NYSE market returns from Jan 1960-Dec 2014,
including the returns on cap based NYSE portfolios 
(https://wrds-web.wharton.upenn.edu/wrds/query_forms/navigation.cfm?navId=124);

data Nyse_cap_portfolios_annual; 
	set input.Nyse_cap_portfolios_annual; 
	drop TOTVAL TOTCNT USDVAL USDCNT SPRTRN SPINDX VWRETD VWRETX EWRETX;
	year=year(CALDT);
run;

* T-bill file: Annual_T_Bill
Downloaded from: https://fred.stlouisfed.org/series/GS1
Note: Use the 1-Year Treasury Constant Maturity Rate (titled variable DGS1_BOY) as issues with 
	  the other 2 alternatives:	Secondary Market Rate (variable TB1YR_BOY) has no data for 2001-2008 
	  (https://research.stlouisfed.org/fred2/series/TB1YR) and Auction Average (variable TB1YA_BOY)
	  is discontinued in 2000 (https://research.stlouisfed.org/fred2/series/TB1YA)

* take market return file, merge in risk free rate and AILLIQ files, then restrict the sample period

Note: returns for Nyse_cap_portfolios_annual are realised returns at end of each year,
	  yields for Annual_T_Bill are quoted at start of each year, to be realised over that year;

Data TS_DATA_EQ_10; 
	merge Nyse_cap_portfolios_annual (in=a) 
		  input.Annual_T_Bill (in=b)
		  Ailliq_ts_reg (in=c);	
	by year; 
	if a; 
run;

data TS_DATA_EQ_10; 
	set TS_DATA_EQ_10; 
	where year >= 1964 and year <= 2015;
	* create variables for excess return;
	RF=DGS1_BOY; 
	RM_RF=EWRETD-RF;
	RS1_RF=CAP1RET-RF; 
	RS2_RF=CAP2RET-RF; 
	RS3_RF=CAP3RET-RF; 
	RS4_RF=CAP4RET-RF; 
	RS5_RF=CAP5RET-RF; 
	RS6_RF=CAP6RET-RF; 
	RS7_RF=CAP7RET-RF; 
	RS8_RF=CAP8RET-RF; 
	RS9_RF=CAP9RET-RF; 
	RS10_RF=CAP10RET-RF; 	
run;


* STEP 5: PERFORM REGRESSION IN EQ. (10): (RM-Rf)y = g0 + g1*lnAILLIQy-1 + g2*lnAILLIQ(U)y + wy
										   where unexpected liquidity [lnAILLIQ(U)y] = v_y_adj;

* Newey West (http://support.sas.com/kb/40/098.html): full-sample so use (bart,3,0);

/************************************************************************************/
							 * DSV Table 4, Panel B ;
/************************************************************************************/

*RM-RF: Table 4, Panel B, column 1;
proc model data=TS_DATA_EQ_10; /*for standard t-statistics*/
         parms b0 b1 b2;
         RM_RF=b0 + b1*ln_AILLIQ_lag + b2*v_y_adj;
         fit RM_RF;
         run;
         quit;
proc model data=TS_DATA_EQ_10; /*for Durbin h-statistic and Newey West t-statistics*/
         endo RM_RF;
         exog ln_AILLIQ_lag v_y_adj;
         instruments _exog_;
         parms b0 b1 b2;
         RM_RF=b0 + b1*ln_AILLIQ_lag + b2*v_y_adj;
         fit RM_RF / gmm kernel=(bart,3,0) vardef=n DW;
         run;
         quit; 


*Size decile 2: Table 4, Panel B, column 2;
proc model data=TS_DATA_EQ_10; /*for standard t-statistics*/
         parms b0 b1 b2;
         RS2_RF=b0 + b1*ln_AILLIQ_lag + b2*v_y_adj;
         fit RS2_RF;
         run;
         quit;
proc model data=TS_DATA_EQ_10; /*for Durbin h-statistic and Newey West t-statistics*/
         endo RS2_RF;
         exog ln_AILLIQ_lag v_y_adj;
         instruments _exog_;
         parms b0 b1 b2;
         RS2_RF=b0 + b1*ln_AILLIQ_lag + b2*v_y_adj;
         fit RS2_RF / gmm kernel=(bart,3,0) vardef=n DW;
         run;
         quit; 


*Size decile 4: Table 4, Panel B, column 3;
proc model data=TS_DATA_EQ_10; /*for standard t-statistics*/
         parms b0 b1 b2;
         RS4_RF=b0 + b1*ln_AILLIQ_lag + b2*v_y_adj;
         fit RS4_RF;
         run;
         quit;
proc model data=TS_DATA_EQ_10; /*for Durbin h-statistic and Newey West t-statistics*/
         endo RS4_RF;
         exog ln_AILLIQ_lag v_y_adj;
         instruments _exog_;
         parms b0 b1 b2;
         RS4_RF=b0 + b1*ln_AILLIQ_lag + b2*v_y_adj;
         fit RS4_RF / gmm kernel=(bart,3,0) vardef=n DW;
         run;
         quit; 


*Size decile 6: Table 4, Panel B, column 4;
proc model data=TS_DATA_EQ_10; /*for standard t-statistics*/
         parms b0 b1 b2;
         RS6_RF=b0 + b1*ln_AILLIQ_lag + b2*v_y_adj;
         fit RS6_RF;
         run;
         quit;
proc model data=TS_DATA_EQ_10; /*for Durbin h-statistic and Newey West t-statistics*/
         endo RS6_RF;
         exog ln_AILLIQ_lag v_y_adj;
         instruments _exog_;
         parms b0 b1 b2;
         RS6_RF=b0 + b1*ln_AILLIQ_lag + b2*v_y_adj;
         fit RS6_RF / gmm kernel=(bart,3,0) vardef=n DW;
         run;
         quit; 


*Size decile 8: Table 4, Panel B, column 5;
proc model data=TS_DATA_EQ_10; /*for standard t-statistics*/
         parms b0 b1 b2;
         RS8_RF=b0 + b1*ln_AILLIQ_lag + b2*v_y_adj;
         fit RS8_RF;
         run;
         quit;
proc model data=TS_DATA_EQ_10; /*for Durbin h-statistic and Newey West t-statistics*/
         endo RS8_RF;
         exog ln_AILLIQ_lag v_y_adj;
         instruments _exog_;
         parms b0 b1 b2;
         RS8_RF=b0 + b1*ln_AILLIQ_lag + b2*v_y_adj;
         fit RS8_RF / gmm kernel=(bart,3,0) vardef=n DW;
         run;
         quit; 


*Size decile 10: Table 4, Panel B, column 6;
proc model data=TS_DATA_EQ_10; /*for standard t-statistics*/
         parms b0 b1 b2;
         RS10_RF=b0 + b1*ln_AILLIQ_lag + b2*v_y_adj;
         fit RS10_RF;
         run;
         quit;
proc model data=TS_DATA_EQ_10; /*for Durbin h-statistic and Newey West t-statistics*/
         endo RS10_RF;
         exog ln_AILLIQ_lag v_y_adj;
         instruments _exog_;
         parms b0 b1 b2;
         RS10_RF=b0 + b1*ln_AILLIQ_lag + b2*v_y_adj;
         fit RS10_RF / gmm kernel=(bart,3,0) vardef=n DW;
         run;
         quit; 



